@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2007 by Rajko Albrecht                             *
+ *   Copyright (C) 2005-2009 by Rajko Albrecht                             *
  *   ral@alwins-world.de                                                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -24,19 +24,21 @@
 #include <QFile>
 #include <QTemporaryFile>
 
-#include "src/svnqt/svnqttypes.hpp"
-#include "src/svnqt/dirent.hpp"
-#include "src/svnqt/url.hpp"
-#include "src/svnqt/status.hpp"
-#include "src/svnqt/targets.hpp"
-#include "src/svnqt/info_entry.hpp"
-#include "src/svnqt/client_parameter.hpp"
-#include "src/svnqt/client_commit_parameter.hpp"
-#include "src/svnqt/shared_pointer.hpp"
+#include "src/svnqt/svnqttypes.h"
+#include "src/svnqt/dirent.h"
+#include "src/svnqt/url.h"
+#include "src/svnqt/status.h"
+#include "src/svnqt/targets.h"
+#include "src/svnqt/info_entry.h"
+#include "src/svnqt/client_parameter.h"
+#include "src/svnqt/client_commit_parameter.h"
+#include "src/svnqt/shared_pointer.h"
 #include "src/settings/kdesvnsettings.h"
 #include "src/helpers/sub2qt.h"
+#include "src/helpers/stringhelper.h"
 #include "src/helpers/sshagent.h"
 #include "kdesvndinterface.h"
+#include "kio_macros.h"
 
 #include <stdlib.h>
 #include <math.h>
@@ -70,6 +72,7 @@ public:
     virtual ~KioSvnData();
 
     void reInitClient();
+    void resetListener();
 
     KioListener m_Listener;
     bool first_done;
@@ -79,6 +82,7 @@ public:
     svn::Client* m_Svnclient;
     svn::Revision urlToRev(const KUrl&);
     QTime _last;
+    qulonglong m_Id;
 };
 
 KioSvnData::KioSvnData(kio_svnProtocol*par)
@@ -89,6 +93,8 @@ KioSvnData::KioSvnData(kio_svnProtocol*par)
     dispProgress = false;
     dispWritten = false;
     _last = QTime::currentTime();
+    // null is an invalid id
+    m_Id = 0;
     reInitClient();
 }
 
@@ -106,6 +112,14 @@ void KioSvnData::reInitClient()
     m_Svnclient->setContext(m_CurrentContext);
 }
 
+void KioSvnData::resetListener()
+{
+    if (!first_done) {
+        reInitClient();
+    }
+    m_Listener.uncancel();
+}
+
 KioSvnData::~KioSvnData()
 {
     m_Listener.setCancel(true);
@@ -142,10 +156,12 @@ kio_svnProtocol::kio_svnProtocol(const QByteArray &pool_socket, const QByteArray
 {
     m_pData=new KioSvnData(this);
     KGlobal::locale()->insertCatalog("kdesvn");
+    m_pData->m_Id = reinterpret_cast<qulonglong>(this);
 }
 
 kio_svnProtocol::~kio_svnProtocol()
 {
+    unregisterFromDaemon();
     delete m_pData;
 }
 
@@ -198,6 +214,7 @@ void kio_svnProtocol::listSendDirEntry(const svn::DirEntry&direntry)
 void kio_svnProtocol::listDir(const KUrl&url)
 {
     kDebug(9510) << "kio_svn::listDir(const KUrl& url) : " << url.url() << endl ;
+    m_pData->resetListener();
     svn::DirEntries dlist;
     svn::Revision rev = m_pData->urlToRev(url);
     if (rev == svn::Revision::UNDEFINED) {
@@ -210,7 +227,7 @@ void kio_svnProtocol::listDir(const KUrl&url)
     } catch (const svn::ClientException&e) {
         QString ex = e.msg();
         kDebug(9510)<<ex<<endl;
-        error(KIO::ERR_SLAVE_DEFINED,ex);
+        extraError(KIO::ERR_SLAVE_DEFINED,ex);
         return;
     }
     listEntry(KIO::UDSEntry(), true );
@@ -221,6 +238,7 @@ void kio_svnProtocol::listDir(const KUrl&url)
 void kio_svnProtocol::stat(const KUrl& url)
 {
     kDebug(9510)<<"kio_svn::stat "<< url << endl;
+    m_pData->resetListener();
     svn::Revision rev = m_pData->urlToRev(url);
     if (rev == svn::Revision::UNDEFINED) {
         rev = svn::Revision::HEAD;
@@ -233,7 +251,7 @@ void kio_svnProtocol::stat(const KUrl& url)
     } catch  (const svn::ClientException&e) {
         QString ex = e.msg();
         kDebug(9510)<<ex<<endl;
-        error( KIO::ERR_SLAVE_DEFINED,ex);
+        extraError( KIO::ERR_SLAVE_DEFINED,ex);
         return;
     }
 
@@ -278,7 +296,11 @@ void kio_svnProtocol::get(const KUrl& url)
         m_pData->m_Svnclient->cat(dstream,_url,rev,rev);
     } catch (const svn::ClientException&e) {
         QString ex = e.msg();
-        error( KIO::ERR_SLAVE_DEFINED,"Subversion error "+ex);
+        // dolphin / Konqueror try to get the content without check if it is a folder when listing a folder
+        // which results in a lot of error messages via kio notify
+        if (e.apr_err()!=SVN_ERR_CLIENT_IS_DIRECTORY) {
+            extraError( KIO::ERR_SLAVE_DEFINED,"Subversion error "+ex);
+        }
         return;
     }
     data(QByteArray()); // empty array means we're done sending the data
@@ -288,12 +310,13 @@ void kio_svnProtocol::get(const KUrl& url)
 void kio_svnProtocol::mkdir(const KUrl &url, int)
 {
     kDebug(9510)<<"kio_svn::mkdir "<< url << endl;
+    m_pData->resetListener();
     svn::Revision rev = m_pData->urlToRev(url);
     if (rev == svn::Revision::UNDEFINED) {
         rev = svn::Revision::HEAD;
     }
     if (rev != svn::Revision::HEAD) {
-        error(KIO::ERR_SLAVE_DEFINED,i18n("Can only write on head revision!"));
+        extraError(KIO::ERR_SLAVE_DEFINED,i18n("Can only write on head revision!"));
         return;
     }
     m_pData->m_CurrentContext->setLogMessage(getDefaultLog());
@@ -301,24 +324,23 @@ void kio_svnProtocol::mkdir(const KUrl &url, int)
         svn::Path p(makeSvnUrl(url));
         m_pData->m_Svnclient->mkdir(p,getDefaultLog());
     }catch (const svn::ClientException&e) {
-        error( KIO::ERR_SLAVE_DEFINED,e.msg());
-        kDebug(9510)<<"kio_svn::mkdir aborted" << url << endl;
+        extraError( KIO::ERR_SLAVE_DEFINED,e.msg());
         return;
     }
-    kDebug(9510)<<"kio_svn::mkdir finished " << url << endl;
     finished();
 }
 
 void kio_svnProtocol::mkdir(const KUrl::List &urls, int)
 {
     svn::Pathes p;
+    m_pData->resetListener();
     for (KUrl::List::const_iterator it = urls.begin(); it != urls.end() ; ++it ) {
         p.append((*it).path());
     }
     try {
         m_pData->m_Svnclient->mkdir(svn::Targets(p),getDefaultLog());
     } catch (const svn::ClientException&e) {
-        error(KIO::ERR_SLAVE_DEFINED,e.msg());
+        extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
         return;
     }
     finished();
@@ -327,6 +349,7 @@ void kio_svnProtocol::mkdir(const KUrl::List &urls, int)
 void kio_svnProtocol::rename(const KUrl&src,const KUrl&target,KIO::JobFlags flags)
 {
     kDebug(9510)<<"kio_svn::rename "<< src << " to " << target <<  endl;
+    m_pData->resetListener();
     Q_UNUSED(flags);
     //bool force  = flags&KIO::Overwrite;
     QString msg;
@@ -337,24 +360,24 @@ void kio_svnProtocol::rename(const KUrl&src,const KUrl&target,KIO::JobFlags flag
         if (e.apr_err()==SVN_ERR_ENTRY_EXISTS) {
             error(KIO::ERR_DIR_ALREADY_EXIST,e.msg());
         } else {
-            error(KIO::ERR_SLAVE_DEFINED,e.msg());
+            extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
         }
-        kDebug(9510)<<"kio_svn::rename aborted" <<  endl;
         return;
     }
-    kDebug(9510)<<"kio_svn::rename finished" <<  endl;
+    notify(i18n("Renaming %1 to %2 succesfull").arg(src.prettyUrl()).arg(target.prettyUrl()));
     finished();
 }
 
 void kio_svnProtocol::put(const KUrl&url,int permissions,KIO::JobFlags flags)
 {
     Q_UNUSED(permissions);
+    m_pData->resetListener();
     svn::Revision rev = m_pData->urlToRev(url);
     if (rev == svn::Revision::UNDEFINED) {
         rev = svn::Revision::HEAD;
     }
     if (rev != svn::Revision::HEAD) {
-        error(KIO::ERR_SLAVE_DEFINED,i18n("Can only write on head revision!"));
+        extraError(KIO::ERR_SLAVE_DEFINED,i18n("Can only write on head revision!"));
         return;
     }
     svn::Revision peg = rev;
@@ -366,7 +389,7 @@ void kio_svnProtocol::put(const KUrl&url,int permissions,KIO::JobFlags flags)
         if (e.apr_err()==SVN_ERR_ENTRY_NOT_FOUND || e.apr_err()==SVN_ERR_RA_ILLEGAL_URL) {
             exists = false;
         } else {
-            error(KIO::ERR_SLAVE_DEFINED,e.msg());
+            extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
             return;
         }
     }
@@ -375,7 +398,7 @@ void kio_svnProtocol::put(const KUrl&url,int permissions,KIO::JobFlags flags)
     if (exists) {
         if (flags & KIO::Overwrite) {
             if (!supportOverwrite()) {
-                error(KIO::ERR_SLAVE_DEFINED,i18n("Overwriting existing items is disabled in settings."));
+                extraError(KIO::ERR_SLAVE_DEFINED,i18n("Overwriting existing items is disabled in settings."));
                 return;
             }
             _codir = new KTempDir;
@@ -383,23 +406,29 @@ void kio_svnProtocol::put(const KUrl&url,int permissions,KIO::JobFlags flags)
             svn::Path path = makeSvnUrl(url.url());
             path.removeLast();
             try {
+                notify(i18n("Start checking out to temporary folder"));
+                m_pData->dispWritten = true;
+                registerToDaemon();
+                startOp(-1, i18n("Checking out %1").arg(path.native()));
                 svn::CheckoutParameter params;
                 params.moduleName(path).destination(svn::Path(_codir->name())).revision(rev).peg(peg).depth(svn::DepthFiles);
                 m_pData->m_Svnclient->checkout(params);
             } catch (const svn::ClientException&e) {
-                error(KIO::ERR_SLAVE_DEFINED,e.msg());
+                extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
                 return;
             }
+            m_pData->dispWritten = false;
+            stopOp(i18n("Temporary checkout done."));
             tmpfile = new QFile(_codir->name()+url.fileName());
             tmpfile->open(QIODevice::ReadWrite|QIODevice::Truncate);
         } else {
-            error(KIO::ERR_FILE_ALREADY_EXIST,i18n("Could not write to existing item."));
+            extraError(KIO::ERR_FILE_ALREADY_EXIST,i18n("Could not write to existing item."));
             return;
         }
     } else {
         KTemporaryFile* _tmpfile = new KTemporaryFile();
         if (!_tmpfile->open()) {
-            error(KIO::ERR_SLAVE_DEFINED,i18n("Could not open temporary file"));
+            extraError(KIO::ERR_SLAVE_DEFINED,i18n("Could not open temporary file"));
             delete _tmpfile;
             return;
         }
@@ -430,6 +459,8 @@ void kio_svnProtocol::put(const KUrl&url,int permissions,KIO::JobFlags flags)
     totalSize(processed_size);
     written (0);
     m_pData->dispWritten = true;
+    registerToDaemon();
+    startOp(processed_size, i18n("Commiting %1").arg(makeSvnUrl(url)));
     bool err = false;
     if (exists) {
         svn::CommitParameter commit_parameters;
@@ -437,7 +468,7 @@ void kio_svnProtocol::put(const KUrl&url,int permissions,KIO::JobFlags flags)
         try {
             m_pData->m_Svnclient->commit(commit_parameters);
         } catch (const svn::ClientException&e) {
-            error(KIO::ERR_SLAVE_DEFINED,e.msg());
+            extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
             err = true;
         }
     } else  {
@@ -445,12 +476,16 @@ void kio_svnProtocol::put(const KUrl&url,int permissions,KIO::JobFlags flags)
             m_pData->m_Svnclient->import(tmpfile->fileName(),makeSvnUrl(url),getDefaultLog(),svn::DepthEmpty,false,false);
         } catch  (const svn::ClientException&e) {
             QString ex = e.msg();
-            error(KIO::ERR_SLAVE_DEFINED,e.msg());
+            extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
             err = true;
+            stopOp(ex);
         }
     }
     m_pData->dispWritten = false;
-    if (!err) finished();
+    if (!err) {
+        stopOp(i18n("Wrote %1 to repository").arg(helpers::ByteToString(processed_size)));
+        finished();
+    }
 }
 
 void kio_svnProtocol::copy(const KUrl&src,const KUrl&dest,int permissions,KIO::JobFlags flags)
@@ -458,7 +493,7 @@ void kio_svnProtocol::copy(const KUrl&src,const KUrl&dest,int permissions,KIO::J
     Q_UNUSED(permissions);
     Q_UNUSED(flags);
     //bool force = flags&KIO::Overwrite;
-
+    m_pData->resetListener();
     kDebug(9510)<<"kio_svn::copy "<< src << " to " << dest <<  endl;
     svn::Revision rev = m_pData->urlToRev(src);
     if (rev == svn::Revision::UNDEFINED) {
@@ -472,19 +507,21 @@ void kio_svnProtocol::copy(const KUrl&src,const KUrl&dest,int permissions,KIO::J
         if (e.apr_err()==SVN_ERR_ENTRY_EXISTS) {
             error(KIO::ERR_DIR_ALREADY_EXIST,e.msg());
         } else {
-            error(KIO::ERR_SLAVE_DEFINED,e.msg());
+            extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
         }
         kDebug(9510)<<"kio_svn::copy aborted" <<  endl;
         return;
     }
     m_pData->dispProgress=false;
     kDebug(9510)<<"kio_svn::copy finished" <<  endl;
+    notify(i18n("Copied %1 to %2").arg(makeSvnUrl(src)).arg(makeSvnUrl(dest)));
     finished();
 }
 
 void kio_svnProtocol::del(const KUrl&src,bool isfile)
 {
     Q_UNUSED(isfile);
+    m_pData->resetListener();
     kDebug(9510)<<"kio_svn::del "<< src << endl;
     //m_pData->reInitClient();
     svn::Revision rev = m_pData->urlToRev(src);
@@ -492,7 +529,7 @@ void kio_svnProtocol::del(const KUrl&src,bool isfile)
         rev = svn::Revision::HEAD;
     }
     if (rev != svn::Revision::HEAD) {
-        error(KIO::ERR_SLAVE_DEFINED,i18n("Can only write on head revision!"));
+        extraError(KIO::ERR_SLAVE_DEFINED,i18n("Can only write on head revision!"));
         return;
     }
     m_pData->m_CurrentContext->setLogMessage(getDefaultLog());
@@ -500,9 +537,7 @@ void kio_svnProtocol::del(const KUrl&src,bool isfile)
         svn::Targets target(makeSvnUrl(src));
         m_pData->m_Svnclient->remove(target,false);
     } catch (const svn::ClientException&e) {
-        QString ex = e.msg();
-        kDebug(9510)<<ex<<endl;
-        error( KIO::ERR_SLAVE_DEFINED,ex);
+        extraError( KIO::ERR_SLAVE_DEFINED,e.msg());
         kDebug(9510)<<"kio_svn::del aborted" << endl;
         return;
     }
@@ -518,6 +553,7 @@ bool kio_svnProtocol::getLogMsg(QString&t)
 
 bool kio_svnProtocol::checkWc(const KUrl&url)
 {
+    m_pData->resetListener();
     if (url.isEmpty()||!url.isLocalFile()) return false;
     svn::Revision peg(svn_opt_revision_unspecified);
     svn::Revision rev(svn_opt_revision_unspecified);
@@ -576,6 +612,7 @@ void kio_svnProtocol::special(const QByteArray& data)
     kDebug(9510)<<"kio_svnProtocol::special"<<endl;
     QByteArray tmpData(data);
     QDataStream stream(&tmpData,QIODevice::ReadOnly);
+    m_pData->resetListener();
     int tmp;
     stream >> tmp;
     kDebug(9510) << "kio_svnProtocol::special " << tmp << endl;
@@ -745,6 +782,7 @@ void kio_svnProtocol::special(const QByteArray& data)
 void kio_svnProtocol::update(const KUrl&url,int revnumber,const QString&revkind)
 {
     svn::Revision where(revnumber,revkind);
+    m_pData->resetListener();
     /* update is always local - so make a path instead URI */
     svn::Path p(url.path());
     try {
@@ -754,7 +792,7 @@ void kio_svnProtocol::update(const KUrl&url,int revnumber,const QString&revkind)
         // sticky depth (last parameter)
         m_pData->m_Svnclient->update(pathes, where,svn::DepthInfinity,false,false,true);
     } catch (const svn::ClientException&e) {
-        error(KIO::ERR_SLAVE_DEFINED,e.msg());
+        extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
     }
 }
 
@@ -762,10 +800,11 @@ void kio_svnProtocol::status(const KUrl&wc,bool cR,bool rec)
 {
     svn::StatusEntries dlist;
     svn::StatusParameter params(wc.path());
+    m_pData->resetListener();
     try {
         dlist = m_pData->m_Svnclient->status(params.depth(rec?svn::DepthInfinity:svn::DepthEmpty).all(false).update(cR).noIgnore(false).revision(svn::Revision::UNDEFINED));
     } catch (const svn::ClientException&e) {
-        error(KIO::ERR_SLAVE_DEFINED,e.msg());
+        extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
         return;
     }
     kDebug(9510)<<"Status got " << dlist.count() << " entries." << endl;
@@ -793,12 +832,7 @@ void kio_svnProtocol::commit(const KUrl::List&url)
     /// @todo replace with direct call to kdesvn?
     QString msg;
 
-    OrgKdeKdesvndInterface kdesvndInterface( "org.kde.kded", "/modules/kdesvnd", QDBusConnection::sessionBus() );
-    if(!kdesvndInterface.isValid())
-    {
-       kWarning() << "Communication with KDED:KdeSvnd failed";
-       return;
-    }
+    CON_DBUS;
 
     QDBusReply<QStringList> res = kdesvndInterface.get_logmsg();
     if ( !res.isValid() )
@@ -825,7 +859,7 @@ void kio_svnProtocol::commit(const KUrl::List&url)
     try {
         nnum = m_pData->m_Svnclient->commit(commit_parameters);
     } catch (const svn::ClientException&e) {
-        error(KIO::ERR_SLAVE_DEFINED,e.msg());
+        extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
     }
     for (long j=0;j<url.count();++j) {
         QString userstring;
@@ -854,7 +888,7 @@ void kio_svnProtocol::checkout(const KUrl&src,const KUrl&target,const int rev, c
         params.moduleName(makeSvnUrl(src)).destination(target.path()).revision(where).peg(svn::Revision::UNDEFINED).depth(svn::DepthInfinity);
         m_pData->m_Svnclient->checkout(params);
     } catch (const svn::ClientException&e) {
-        error(KIO::ERR_SLAVE_DEFINED,e.msg());
+        extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
     }
 }
 
@@ -870,7 +904,7 @@ void kio_svnProtocol::svnlog(int revstart,const QString&revstringstart,int reven
         try {
             m_pData->m_Svnclient->log(params.targets(makeSvnUrl(urls[j])),logs);
         } catch (const svn::ClientException&e) {
-            error(KIO::ERR_SLAVE_DEFINED,e.msg());
+            extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
             break;
         }
         if (logs.size()==0) {
@@ -920,7 +954,7 @@ void kio_svnProtocol::revert(const KUrl::List&l)
     try {
         m_pData->m_Svnclient->revert(target,svn::DepthEmpty);
     } catch (const svn::ClientException&e) {
-        error(KIO::ERR_SLAVE_DEFINED,e.msg());
+        extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
     }
 }
 
@@ -931,7 +965,7 @@ void kio_svnProtocol::wc_switch(const KUrl&wc,const KUrl&target,bool rec,int rev
     try {
         m_pData->m_Svnclient->doSwitch(wc_path,makeSvnUrl(target.url()),where,rec?svn::DepthInfinity:svn::DepthFiles);
     } catch (const svn::ClientException&e) {
-        error(KIO::ERR_SLAVE_DEFINED,e.msg());
+        extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
     }
 }
 
@@ -958,7 +992,7 @@ void kio_svnProtocol::diff(const KUrl&uri1,const KUrl&uri2,int rnum1,const QStri
         tdir.setAutoRemove(true);
         ex = m_pData->m_Svnclient->diff(_opts);
     } catch (const svn::ClientException&e) {
-        error(KIO::ERR_SLAVE_DEFINED,e.msg());
+        extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
         return;
     }
     QString out = QString::FROMUTF8(ex);
@@ -976,7 +1010,7 @@ void kio_svnProtocol::import(const KUrl& repos, const KUrl& wc)
         QString path = wc.path();
         m_pData->m_Svnclient->import(svn::Path(path),target,QString(),svn::DepthInfinity,false,false);
     } catch (const svn::ClientException&e) {
-        error(KIO::ERR_SLAVE_DEFINED,e.msg());
+        extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
         return;
     }
     finished();
@@ -989,7 +1023,7 @@ void kio_svnProtocol::add(const KUrl& wc)
                                                /* rec */
         m_pData->m_Svnclient->add(svn::Path(path),svn::DepthInfinity);
     } catch (const svn::ClientException&e) {
-        error(KIO::ERR_SLAVE_DEFINED,e.msg());
+        extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
         return;
     }
     finished();
@@ -1004,7 +1038,7 @@ void kio_svnProtocol::wc_delete(const KUrl::List&l)
     try {
         m_pData->m_Svnclient->remove(svn::Targets(p),false);
     } catch (const svn::ClientException&e) {
-        error(KIO::ERR_SLAVE_DEFINED,e.msg());
+        extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
         return;
     }
     finished();
@@ -1016,7 +1050,7 @@ void kio_svnProtocol::wc_resolve(const KUrl&url,bool recurse)
         svn::Depth depth=recurse?svn::DepthInfinity:svn::DepthEmpty;
         m_pData->m_Svnclient->resolve(url.path(),depth);
     } catch (const svn::ClientException&e) {
-        error(KIO::ERR_SLAVE_DEFINED,e.msg());
+        extraError(KIO::ERR_SLAVE_DEFINED,e.msg());
         return;
     }
     finished();
@@ -1045,17 +1079,26 @@ void kio_svnProtocol::contextProgress(long long int current, long long int max)
         totalSize(KIO::filesize_t(max));
     }
 
-    if (m_pData->dispProgress||m_pData->dispWritten) {
+    bool to_dbus = false;
+    if (m_pData->dispProgress||m_pData->dispWritten||max > -1) {
         QTime now = QTime::currentTime();
         if (m_pData->_last.msecsTo(now)>=90) {
             if (m_pData->dispProgress) {
                 processedSize(KIO::filesize_t(current));
             } else {
                 written(current);
+                to_dbus = true;
             }
             m_pData->_last = now;
         }
     }
+    if (to_dbus) {
+        CON_DBUS;
+        if (max > -1) {
+            kdesvndInterface.maxTransferKioOperation(m_pData->m_Id,max);
+        }
+        kdesvndInterface.transferedKioOperation(m_pData->m_Id,current);
+    }
 }
 
 bool kio_svnProtocol::supportOverwrite()
@@ -1077,4 +1120,52 @@ QString kio_svnProtocol::getDefaultLog()
     return res;
 }
 
+void kio_svnProtocol::notify(const QString&text)
+{
+    CON_DBUS;
+    kdesvndInterface.notifyKioOperation(text);
+}
+
+void kio_svnProtocol::extraError(int _errid,const QString&text)
+{
+    error(_errid,text);
+    if (!text.isNull()) {
+        CON_DBUS;
+        kdesvndInterface.errorKioOperation(text);
+    }
+}
+
+void kio_svnProtocol::registerToDaemon()
+{
+    CON_DBUS;
+    kdesvndInterface.registerKioFeedback(m_pData->m_Id);
+}
+
+void kio_svnProtocol::unregisterFromDaemon()
+{
+    CON_DBUS;
+    kdesvndInterface.unRegisterKioFeedback(m_pData->m_Id);
+}
+bool kio_svnProtocol::checkKioCancel()const
+{
+    CON_DBUS_VAL(false);
+    QDBusReply<bool> res=kdesvndInterface.canceldKioOperation(m_pData->m_Id);
+    return res.isValid()?res.value():false;
+}
+
+void kio_svnProtocol::startOp(qulonglong max, const QString&title)
+{
+    CON_DBUS;
+    kdesvndInterface.maxTransferKioOperation(m_pData->m_Id,max);
+    kdesvndInterface.titleKioOperation(m_pData->m_Id,title,title);
+    kdesvndInterface.setKioStatus(m_pData->m_Id,1,QString());
+}
+
+void kio_svnProtocol::stopOp(const QString&message)
+{
+    CON_DBUS;
+    kdesvndInterface.setKioStatus(m_pData->m_Id,0,message);
+    unregisterFromDaemon();
+}
+
 } // namespace KIO
