@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2007 by Rajko Albrecht                             *
+ *   Copyright (C) 2005-2009 by Rajko Albrecht                             *
  *   ral@alwins-world.de                                                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,9 +21,9 @@
 #include "src/settings/kdesvnsettings.h"
 #include "svnfrontend/svnactions.h"
 #include "svnfrontend/dummydisplay.h"
-#include "src/svnqt/targets.hpp"
-#include "src/svnqt/url.hpp"
-#include "src/svnqt/dirent.hpp"
+#include "src/svnqt/targets.h"
+#include "src/svnqt/url.h"
+#include "src/svnqt/dirent.h"
 #include "src/helpers/sub2qt.h"
 #include "src/helpers/ktranslateurl.h"
 #include "src/helpers/sshagent.h"
@@ -427,7 +427,7 @@ void CommandExec::slotCmd_get()
 void CommandExec::slotCmd_update()
 {
     m_pCPart->m_SvnWrapper->makeUpdate(m_pCPart->url,
-        (m_pCPart->rev_set?m_pCPart->start:svn::Revision::HEAD),true);
+        (m_pCPart->rev_set?m_pCPart->start:svn::Revision::HEAD),svn::DepthUnknown);
 }
 
 void CommandExec::slotCmd_diff()
