@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2007 by Rajko Albrecht                             *
+ *   Copyright (C) 2005-2009 by Rajko Albrecht                             *
  *   ral@alwins-world.de                                                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -21,7 +21,7 @@
 #include "itemdisplay.h"
 #include "svnitem.h"
 #include "src/settings/kdesvnsettings.h"
-#include "src/svnqt/status.hpp"
+#include "src/svnqt/status.h"
 
 
 ItemDisplay::ItemDisplay()
@@ -61,7 +61,7 @@ void ItemDisplay::setBaseUri(const QString&uri)
 {
     m_baseUri = uri;
     /* otherwise subversion lib asserts! */
-    while (m_baseUri.endsWith("/")) {
+    while (m_baseUri.endsWith('/')) {
         m_baseUri.truncate(m_baseUri.length()-1);
     }
 }
