@@ -32,11 +32,14 @@
 #include "importdir_logmsg.h"
 #include "src/settings/kdesvnsettings.h"
 #include "helpers/sshagent.h"
-#include "src/svnqt/url.hpp"
+#include "src/svnqt/url.h"
+#include "src/svnqt/svnqttypes.h"
 #include "fronthelpers/createdlg.h"
 #include "fronthelpers/rangeinput_impl.h"
 #include "fronthelpers/widgetblockstack.h"
+#include "fronthelpers/fronthelpers.h"
 #include "src/ksvnwidgets/commitmsg_impl.h"
+#include "src/ksvnwidgets/deleteform_impl.h"
 #include "opencontextmenu.h"
 
 #include <kmessagebox.h>
@@ -51,6 +54,7 @@
 #include <kio/deletejob.h>
 #include <kio/copyjob.h>
 #include <kfiledialog.h>
+#include <knotification.h>
 
 #include <QSortFilterProxyModel>
 #include <QEvent>
@@ -107,6 +111,8 @@ public:
     SvnDirSortFilterProxy*m_DirSortModel;
     svn::Revision m_remoteRevision;
     QString merge_Target,merge_Src2,merge_Src1;
+
+    QTimer m_TimeModified,m_TimeUpdates;
 };
 
 MainTreeWidget::MainTreeWidget(KActionCollection*aCollection,QWidget*parent,Qt::WindowFlags f)
@@ -155,6 +161,8 @@ MainTreeWidget::MainTreeWidget(KActionCollection*aCollection,QWidget*parent,Qt::
     connect(m_Data->m_Model->svnWrapper(),SIGNAL(sigThreadsChanged()),this,SLOT(enableActions()));
     connect(m_Data->m_Model->svnWrapper(),SIGNAL(sigCacheDataChanged()),this,SLOT(slotCacheDataChanged()));
 
+    connect(m_Data->m_Model->svnWrapper(),SIGNAL(sigExtraStatusMessage(const QString&)),this,SIGNAL(sigExtraStatusMessage(const QString&)));
+
     connect(m_Data->m_Model,SIGNAL(urlDropped(const KUrl::List&,Qt::DropAction,const QModelIndex &,bool)),
             this,SLOT(slotUrlDropped(const KUrl::List&,Qt::DropAction,const QModelIndex &,bool)));
 
@@ -165,6 +173,11 @@ MainTreeWidget::MainTreeWidget(KActionCollection*aCollection,QWidget*parent,Qt::
 
     checkUseNavigation(true);
     setupActions();
+
+    m_Data->m_TimeModified.setParent(this);
+    connect(&(m_Data->m_TimeModified),SIGNAL(timeout()),this,SLOT(slotCheckModified()));
+    m_Data->m_TimeUpdates.setParent(this);
+    connect(&(m_Data->m_TimeUpdates),SIGNAL(timeout()),this,SLOT(slotCheckUpdates()));
 }
 
 MainTreeWidget::~MainTreeWidget()
@@ -189,6 +202,12 @@ void MainTreeWidget::resizeAllColumns()
 
 bool MainTreeWidget::openUrl(const KUrl &url,bool noReinit)
 {
+
+#ifdef DEBUG_TIMER
+    QTime _counttime;
+    _counttime.start();
+#endif
+
     CursorStack a;
     m_Data->m_Model->svnWrapper()->killallThreads();
     clear();
@@ -295,16 +314,34 @@ bool MainTreeWidget::openUrl(const KUrl &url,bool noReinit)
              slotCheckUpdates();
         }
     }
+#ifdef DEBUG_TIMER
+    _counttime.restart();
+#endif
+
     if (result && Kdesvnsettings::log_cache_on_open()) {
         m_Data->m_Model->svnWrapper()->startFillCache(baseUri(),true);
     }
-
+#ifdef DEBUG_TIMER
+    kDebug()<<"Starting cache "<<_counttime.elapsed();
+    _counttime.restart();
+#endif
     emit changeCaption(baseUri());
     emit sigUrlOpend(result);
     emit sigUrlChanged(baseUri());
+#ifdef DEBUG_TIMER
+    kDebug()<<"Fired signals "<<_counttime.elapsed();
+    _counttime.restart();
+#endif
 
     QTimer::singleShot(1,this,SLOT(readSupportData()));
     enableActions();
+#ifdef DEBUG_TIMER
+    kDebug()<<"Enabled actions "<<_counttime.elapsed();
+#endif
+/*  KNotification * notification=new KNotification("kdesvn-open");
+    notification->setText("Opened url");
+    notification->sendEvent();
+*/
     return result;
 }
 
@@ -468,7 +505,8 @@ void MainTreeWidget::setupActions()
     tmp_action = add_action("make_svn_partialtree",i18n("Partial revision tree"),KShortcut(Qt::SHIFT+Qt::CTRL+Qt::Key_T),KIcon("kdesvntree"),this,SLOT(slotMakePartTree()));
     tmp_action->setStatusTip(i18n("Shows history of item as linked tree for a revision range"));
 
-    tmp_action = add_action("make_svn_property",i18n("Properties"),KShortcut(Qt::CTRL+Qt::Key_P),KIcon(),m_Data->m_Model->svnWrapper(),SLOT(slotProperties()));
+    tmp_action = add_action("make_svn_property",i18n("Properties"),KShortcut(Qt::CTRL+Qt::Key_P),KIcon(),this,SLOT(slotRightProperties()));
+    tmp_action = add_action("make_left_svn_property",i18n("Properties"),KShortcut(),KIcon(),this,SLOT(slotLeftProperties()));
     add_action("get_svn_property",i18n("Display Properties"),KShortcut(Qt::SHIFT+Qt::CTRL+Qt::Key_P),KIcon(),this,SLOT(slotDisplayProperties()));
     tmp_action = add_action("make_last_change",i18n("Display last changes"),KShortcut(),KIcon("kdesvndiff"),this,SLOT(slotDisplayLastDiff()));
     tmp_action->setToolTip(i18n("Display last changes as difference to previous commit."));
@@ -543,6 +581,9 @@ void MainTreeWidget::setupActions()
     tmp_action = add_action("make_svn_remove",i18n("Delete selected files/dirs"),KShortcut(Qt::Key_Delete),KIcon("kdesvndelete"),this,SLOT(slotDelete()));
     tmp_action->setIconText(i18n("Delete"));
     tmp_action->setToolTip(i18n("Deleting selected files and/or directories from repository"));
+    tmp_action = add_action("make_svn_remove_left",i18n("Delete folder"),KShortcut(),KIcon("kdesvndelete"),this,SLOT(slotLeftDelete()));
+    tmp_action->setToolTip(i18n("Deleting selected directories from repository"));
+    tmp_action->setIconText(i18n("Delete"));
     tmp_action  = add_action("make_svn_revert",i18n("Revert current changes"),KShortcut(),KIcon("kdesvnreverse"),m_Data->m_Model->svnWrapper(),SLOT(slotRevert()));
 
     tmp_action = add_action("make_resolved",i18n("Mark resolved"),KShortcut(),KIcon("kdesvnresolved"),this,SLOT(slotResolved()));
@@ -673,8 +714,10 @@ void MainTreeWidget::enableActions()
     enableAction("make_svn_partialtree",single||none);
 
     enableAction("make_svn_property",single);
+    enableAction("make_left_svn_property",d==1);
     enableAction("get_svn_property",single);
     enableAction("make_svn_remove",(multi||single));
+    enableAction("make_svn_remove_left",d>0);
     enableAction("make_svn_lock",(multi||single));
     enableAction("make_svn_unlock",(multi||single));
 
@@ -692,7 +735,7 @@ void MainTreeWidget::enableActions()
     enableAction("make_svn_mkdir",dir||(none && isopen));
     enableAction("make_svn_switch",isWorkingCopy()&& (single||none));
     enableAction("make_switch_to_repo",isWorkingCopy());
-    enableAction("make_import_dirs_into_current",dir);
+    enableAction("make_import_dirs_into_current",dir||d==1);
     enableAction("make_svn_relocate",isWorkingCopy()&& (single||none));
 
     enableAction("make_svn_export_current",((single&&dir)||none));
@@ -874,7 +917,18 @@ void MainTreeWidget::itemActivated(const QModelIndex&index,bool keypress)
 
 void MainTreeWidget::slotCheckUpdates()
 {
-    m_Data->m_Model->svnWrapper()->createUpdateCache(baseUri());
+    if (isWorkingCopy() && m_Data->m_Model->svnWrapper()->doNetworking()) {
+        m_Data->m_TimeUpdates.stop();
+        m_Data->m_Model->svnWrapper()->createUpdateCache(baseUri());
+    }
+}
+
+void MainTreeWidget::slotCheckModified()
+{
+    if (isWorkingCopy()) {
+        m_Data->m_TimeModified.stop();
+        m_Data->m_Model->svnWrapper()->createModifiedCache(baseUri());
+    }
 }
 
 void MainTreeWidget::slotNotifyMessage(const QString&what)
@@ -898,6 +952,16 @@ void MainTreeWidget::slotCacheDataChanged()
 {
     //kDebug(9510)<<"Cache thread signaled"<<endl;
     m_Data->m_SortModel->invalidate();
+    if (isWorkingCopy()) {
+        if (!m_Data->m_TimeModified.isActive() && Kdesvnsettings::poll_modified()) {
+            m_Data->m_TimeModified.setInterval(MinutesToMsec(Kdesvnsettings::poll_modified_minutes()));
+            m_Data->m_TimeModified.start();
+        }
+        if (!m_Data->m_TimeUpdates.isActive() && Kdesvnsettings::poll_updates()) {
+            m_Data->m_TimeUpdates.setInterval(MinutesToMsec(Kdesvnsettings::poll_updates_minutes()));
+            m_Data->m_TimeUpdates.start();
+        }
+    }
 }
 
 void MainTreeWidget::slotIgnore()
@@ -964,6 +1028,8 @@ void MainTreeWidget::slotDirContextMenu(const QPoint&vp)
     if ( (temp=filesActions()->action("make_svn_dirbasediff")) && temp->isEnabled() && ++count) popup.addAction(temp);
     if ( (temp=filesActions()->action("make_svn_diritemsdiff")) && temp->isEnabled() && ++count) popup.addAction(temp);
     if ( (temp=filesActions()->action("make_svn_dir_log_nofollow")) && temp->isEnabled() && ++count) popup.addAction(temp);
+    if ( (temp=filesActions()->action("make_left_svn_property")) && temp->isEnabled() && ++count) popup.addAction(temp);
+    if ( (temp=filesActions()->action("make_svn_remove_left")) && temp->isEnabled() && ++count) popup.addAction(temp);
 
     KService::List offers;
     OpenContextmenu*me=0;
@@ -1089,7 +1155,7 @@ void MainTreeWidget::slotSelectBrowsingRevision()
 {
     if (isWorkingCopy()) return;
     Rangeinput_impl*rdlg = 0;
-    svn::SharedPointer<KDialog> dlg = createDialog(&rdlg,QString(i18n("Revisions")),true,"revisions_dlg");
+    svn::SharedPointer<KDialog> dlg = createOkDialog(&rdlg,QString(i18n("Revisions")),true,"revisions_dlg");
     if (!dlg) {
         return;
     }
@@ -1134,7 +1200,7 @@ void MainTreeWidget::slotMakePartTree()
         return;
     }
     Rangeinput_impl*rdlg = 0;
-    svn::SharedPointer<KDialog> dlg = createDialog(&rdlg,QString(i18n("Revisions")),true,"revisions_dlg");
+    svn::SharedPointer<KDialog> dlg = createOkDialog(&rdlg,QString(i18n("Revisions")),true,"revisions_dlg");
     if (!dlg) {
         return;
     }
@@ -1161,7 +1227,7 @@ void MainTreeWidget::slotLock()
         return;
     }
     Commitmsg_impl*ptr = 0;
-    svn::SharedPointer<KDialog> dlg = createDialog(&ptr,QString(i18n("Lock message")),true,"locking_log_msg");
+    svn::SharedPointer<KDialog> dlg = createOkDialog(&ptr,QString(i18n("Lock message")),true,"locking_log_msg");
     if (!dlg) return;
     ptr->initHistory();
     ptr->hideDepth(true);
@@ -1306,7 +1372,7 @@ void MainTreeWidget::slotDiffRevisions()
         what = relativePath(k);
     }
     Rangeinput_impl*rdlg = 0;
-    KDialog*dlg = createDialog(&rdlg,QString(i18n("Revisions")),true,"revisions_dlg");
+    KDialog*dlg = createOkDialog(&rdlg,QString(i18n("Revisions")),true,"revisions_dlg");
     if (!dlg) {
         return;
     }
@@ -1393,7 +1459,7 @@ void MainTreeWidget::slotRangeBlame()
     SvnItem*k = Selected();
     if (!k) return;
     Rangeinput_impl*rdlg = 0;
-    svn::SharedPointer<KDialog> dlg = createDialog(&rdlg,QString(i18n("Revisions")),true,"revisions_dlg");
+    svn::SharedPointer<KDialog> dlg = createOkDialog(&rdlg,QString(i18n("Revisions")),true,"revisions_dlg");
     if (!dlg) {
         return;
     }
@@ -1463,7 +1529,7 @@ void MainTreeWidget::slotRevisionCat()
     SvnItem*k = Selected();
     if (!k) return;
     Rangeinput_impl*rdlg = 0;
-    KDialog*dlg = createDialog(&rdlg,QString(i18n("Revisions")),true,"revisions_dlg");
+    KDialog*dlg = createOkDialog(&rdlg,QString(i18n("Revisions")),true,"revisions_dlg");
     if (!dlg) {
         return;
     }
@@ -1497,13 +1563,22 @@ void MainTreeWidget::slotTryResolve()
     m_Data->m_Model->svnWrapper()->slotResolve(which->fullName());
 }
 
+void MainTreeWidget::slotLeftDelete()
+{
+    SvnItemList lst;
+    DirSelectionList(lst);
+    makeDelete(lst);
+}
+
 void MainTreeWidget::slotDelete()
 {
-    //m_deletePerfect = true;
-
     SvnItemList lst;
     SelectionList(lst);
+    makeDelete(lst);
+}
 
+void MainTreeWidget::makeDelete(const SvnItemList&lst)
+{
     if (lst.size()==0) {
         KMessageBox::error(this,i18n("Nothing selected for delete"));
         return;
@@ -1521,22 +1596,35 @@ void MainTreeWidget::slotDelete()
         }
         displist.append((*liter)->fullName());
     }
-    int answer = KMessageBox::questionYesNoList(this,i18n("Really delete these entries?"),displist,i18n("Delete from repository"));
-    if (answer!=KMessageBox::Yes) {
+
+    DeleteForm_impl*ptr = 0;
+    KDialog*dlg = createYesDialog(&ptr,QString(i18n("Really delete these entries?")),true,"delete_items_dialog",true);
+    if (!dlg) {
         return;
     }
-    WidgetBlockStack st(this);
-    if (kioList.count()>0) {
-        KIO::Job*aJob = KIO::del(kioList);
-	if (!aJob->exec()) {
-	  aJob->showErrorDialog(this);
-	  return;
-	}
+    ptr->setStringList(displist);
+    ptr->showExtraButtons(isWorkingCopy() && items.size()>0);
+    int _ans = dlg->exec();
+    KConfigGroup _kc(Kdesvnsettings::self()->config(),"delete_items_dialog");
+    dlg->saveDialogSize(_kc);
+    bool force = ptr->force_delete();
+    bool keep = ptr->keep_local();
+    delete dlg;
+
+    if (_ans == KDialog::Yes) {
+        WidgetBlockStack st(this);
+        if (kioList.count()>0) {
+            KIO::Job*aJob = KIO::del(kioList);
+            if (!aJob->exec()) {
+                aJob->showErrorDialog(this);
+                return;
+            }
+        }
+        if (items.size()>0) {
+            m_Data->m_Model->svnWrapper()->makeDelete(items,keep,force);
+        }
+        refreshCurrentTree();
     }
-    if (items.size()>0) {
-        m_Data->m_Model->svnWrapper()->makeDelete(items);
-    }
-    refreshCurrentTree();
 }
 
 void MainTreeWidget::internalDrop(const KUrl::List&_lst,Qt::DropAction action,const QModelIndex&index)
@@ -1795,10 +1883,10 @@ void MainTreeWidget::slotMerge()
         target = m_Data->merge_Target;
     }
     src2 = m_Data->merge_Src2;
-    bool force,dry,rec,irelated,useExternal;
+    bool force,dry,rec,irelated,useExternal,recordOnly,reintegrate;
     Rangeinput_impl::revision_range range;
     MergeDlg_impl*ptr = 0;
-    KDialog*dlg = createDialog(&ptr,QString(i18n("Merge")),true,"merge_dialog",true);
+    KDialog*dlg = createOkDialog(&ptr,QString(i18n("Merge")),true,"merge_dialog",true);
     if (!dlg) {
         return;
     }
@@ -1821,11 +1909,13 @@ void MainTreeWidget::slotMerge()
         rec = ptr->recursive();
         irelated = ptr->ignorerelated();
         useExternal = ptr->useExtern();
+        recordOnly = ptr->recordOnly();
         range = ptr->getRange();
+        reintegrate=ptr->reintegrate();
         if (!useExternal) {
             m_Data->m_Model->svnWrapper()->slotMerge(src1,src2,target,range.first,range.second,
                                                       isWorkingCopy()?svn::Revision::UNDEFINED:m_Data->m_remoteRevision,
-                                                      rec,!irelated,force,dry);
+                                                      rec,!irelated,force,dry,recordOnly,reintegrate);
         } else {
             m_Data->m_Model->svnWrapper()->slotMergeExternal(src1,src2,target,range.first,range.second,
                                                               isWorkingCopy()?svn::Revision::UNDEFINED:m_Data->m_remoteRevision,
@@ -1857,7 +1947,7 @@ void MainTreeWidget::slotRelocate()
     path = k->fullName();
     fromUrl = k->Url();
     CheckoutInfo_impl*ptr = 0;
-    KDialog * dlg = createDialog(&ptr,i18n("Relocate path %1",path),true,"relocate_dlg");
+    KDialog * dlg = createOkDialog(&ptr,i18n("Relocate path %1",path),true,"relocate_dlg");
     if (dlg) {
         ptr->setStartUrl(fromUrl);
         ptr->disableAppend(true);
@@ -1925,11 +2015,11 @@ void MainTreeWidget::slotImportIntoDir(const KUrl&importUrl,const QString&target
     }
 
     if (dirs) {
-        dlg = createDialog(&ptr2,QString(i18n("Import log")),true,"import_log_msg");
+        dlg = createOkDialog(&ptr2,QString(i18n("Import log")),true,"import_log_msg");
         ptr = ptr2;
         ptr2->createDirboxDir("\""+uri.fileName()+"\"");
     } else {
-        dlg = createDialog(&ptr,QString(i18n("Import log")),true,"import_log_msg");
+        dlg = createOkDialog(&ptr,QString(i18n("Import log")),true,"import_log_msg");
     }
 
     if (!dlg) return;
@@ -2126,7 +2216,7 @@ void MainTreeWidget::slotDirUpdate()
             what.append((*liter)->fullName());
         }
     }
-    m_Data->m_Model->svnWrapper()->makeUpdate(what,svn::Revision::HEAD,true);
+    m_Data->m_Model->svnWrapper()->makeUpdate(what,svn::Revision::HEAD,svn::DepthUnknown);
 }
 
 void MainTreeWidget::slotRescanIcons()
@@ -2177,4 +2267,18 @@ void MainTreeWidget::slotRepositorySettings()
     }
 }
 
+void MainTreeWidget::slotRightProperties()
+{
+    SvnItem*k = Selected();
+    if (!k) return;
+    m_Data->m_Model->svnWrapper()->editProperties(k,isWorkingCopy()?svn::Revision::WORKING:svn::Revision::HEAD);
+}
+
+void MainTreeWidget::slotLeftProperties()
+{
+    SvnItem*k = DirSelected();
+    if (!k) return;
+    m_Data->m_Model->svnWrapper()->editProperties(k,isWorkingCopy()?svn::Revision::WORKING:svn::Revision::HEAD);
+}
+
 #include "maintreewidget.moc"
