@@ -21,13 +21,14 @@
 #include "svnitemmodel.h"
 #include "svnitemnode.h"
 #include "svnactions.h"
+#include "getinfothread.h"
 #include "src/svnfrontend/maintreewidget.h"
 #include "src/settings/kdesvnsettings.h"
 
-#include "src/svnqt/status.hpp"
-#include "src/svnqt/client.hpp"
-#include "src/svnqt/path.hpp"
-#include "src/svnqt/svnqt_defines.hpp"
+#include "src/svnqt/status.h"
+#include "src/svnqt/client.h"
+#include "src/svnqt/path.h"
+#include "src/svnqt/svnqt_defines.h"
 
 #include <klocale.h>
 #include <kdebug.h>
@@ -39,6 +40,8 @@
 #include <QFileInfo>
 #include <QBrush>
 #include <QUuid>
+#include <QMimeData>
+#include <QDir>
 
 /*****************************
  * Internal data class begin *
@@ -50,10 +53,17 @@ public:
         :m_rootNode(0),m_Cb(aCb),m_Display(display),m_DirWatch(0)
     {
         m_Uid = QUuid::createUuid().toString();
+        m_InfoThread = new GetInfoThread(aCb);
     }
 
     virtual ~SvnItemModelData()
     {
+        m_InfoThread->cancelMe();
+        if (!m_InfoThread->wait(500)) {
+            m_InfoThread->terminate();
+        }
+        delete m_InfoThread;
+
         delete m_rootNode;
         delete m_DirWatch;
         m_rootNode=0;
@@ -117,6 +127,7 @@ public:
     MainTreeWidget*m_Display;
     KDirWatch*m_DirWatch;
     QString m_Uid;
+    mutable GetInfoThread*m_InfoThread;
 };
 /*****************************
  * Internal data class end   *
@@ -165,6 +176,16 @@ void SvnItemModel::clear()
     endRemoveRows();
 }
 
+void SvnItemModel::beginRemoveRows( const QModelIndex & parent, int first, int last )
+{
+    m_Data->m_InfoThread->clearNodes();
+    m_Data->m_InfoThread->cancelMe();
+    if (!m_Data->m_InfoThread->wait(1000)) {
+
+    }
+    QAbstractItemModel::beginRemoveRows(parent,first,last);
+}
+
 void SvnItemModel::clearNodeDir(SvnItemModelNodeDir*node)
 {
     QModelIndex ind = m_Data->indexForNode(node);
@@ -262,7 +283,12 @@ QVariant SvnItemModel::data(const QModelIndex&index,int role)const
         {
             switch(index.column()) {
                 case Name:
-                    return node->getToolTipText();
+                    if (node->hasToolTipText()) {
+                        return node->getToolTipText();
+                    } else {
+                        m_Data->m_InfoThread->appendNode(node);
+                        return QVariant();
+                    }
             }
         }
     }
@@ -316,6 +342,10 @@ int SvnItemModel::columnCount(const QModelIndex & /*parent*/)const
 
 int SvnItemModel::rowCount(const QModelIndex & parent)const
 {
+    if (!m_Data || !m_Data->m_rootNode) {
+        return 0;
+    }
+
     if (!parent.isValid()) {
         return m_Data->m_rootNode->childList().count();
     }
@@ -347,6 +377,11 @@ int SvnItemModel::checkDirs(const QString&_what,SvnItemModelNode*_parent)
         what.truncate(what.length()-1);
     }
     // prevent this from checking unversioned folder. FIXME: what happen when we do open url on a non-working-copy folder??
+#ifdef DEBUG_TIMER
+    QTime _counttime;
+    _counttime.start();
+#endif
+
     if (!m_Data->m_Display->isWorkingCopy()|| (!_parent) || ((_parent) && (_parent->isVersioned()))) {
         if (!svnWrapper()->makeStatus(what,dlist,m_Data->m_Display->baseRevision(),false,true,true) ) {
             return -1;
@@ -354,6 +389,10 @@ int SvnItemModel::checkDirs(const QString&_what,SvnItemModelNode*_parent)
     } else {
         return checkUnversionedDirs(_parent);
     }
+#ifdef DEBUG_TIMER
+    kDebug()<<"Time for getting entries: "<<_counttime.elapsed();
+    _counttime.restart();
+#endif
     svn::StatusEntries neweritems;
     svnWrapper()->getaddedItems(what,neweritems);
     dlist+=neweritems;
@@ -382,6 +421,9 @@ int SvnItemModel::checkDirs(const QString&_what,SvnItemModelNode*_parent)
     if (_parent) {
         node=_parent;
     }
+#ifdef DEBUG_TIMER
+    kDebug()<<"Time finding parent node: "<<_counttime.elapsed();
+#endif
     insertDirs(node,dlist);
     return dlist.size();
 }
@@ -401,13 +443,24 @@ void SvnItemModel::insertDirs(SvnItemModelNode*_parent,svn::StatusEntries&dlist)
     SvnItemModelNode*node = 0;
     beginInsertRows(ind,parent->childList().count(),parent->childList().count()+dlist.count()-1);
     svn::StatusEntries::iterator it = dlist.begin();
+#ifdef DEBUG_TIMER
+    QTime _counttime;
+    _counttime.start();
+#endif
     for (;it!=dlist.end();++it) {
+#ifdef DEBUG_TIMER
+        _counttime.restart();
+#endif
         if (m_Data->MustCreateDir(*(*it))) {
             node=new SvnItemModelNodeDir(parent,svnWrapper(),m_Data->m_Display);
         } else {
             node=new SvnItemModelNode(parent,svnWrapper(),m_Data->m_Display);
         }
         node->setStat((*it));
+#ifdef DEBUG_TIMER
+        kDebug()<<"Time creating item: "<<_counttime.elapsed();
+        _counttime.restart();
+#endif
         if (m_Data->m_Display->isWorkingCopy() && m_Data->m_DirWatch) {
             if (node->isDir()) {
                 m_Data->addWatchDir(node->fullName());
@@ -415,9 +468,22 @@ void SvnItemModel::insertDirs(SvnItemModelNode*_parent,svn::StatusEntries&dlist)
                 m_Data->addWatchFile(node->fullName());
             }
         }
+#ifdef DEBUG_TIMER
+        kDebug()<<"Time add watch: "<<_counttime.elapsed();
+        _counttime.restart();
+#endif
         parent->m_Children.append(node);
+#ifdef DEBUG_TIMER
+        kDebug()<<"Time append node: "<<_counttime.elapsed();
+#endif
     }
+#ifdef DEBUG_TIMER
+    _counttime.restart();
+#endif
     endInsertRows();
+#ifdef DEBUG_TIMER
+    kDebug()<<"Time append all node: "<<_counttime.elapsed();
+#endif
 }
 
 bool SvnItemModel::canFetchMore(const QModelIndex & parent)const
@@ -732,7 +798,6 @@ bool SvnItemModel::refreshCurrentTree()
         check_created=true;
     }
     return refreshDirnode(_start,check_created);
-    //emit dataChanged(QModelIndex(),0,m_Data->m_rootNode->m_Children.size());
 }
 
 bool SvnItemModel::refreshDirnode(SvnItemModelNodeDir*node,bool check_empty,bool notrec)
@@ -863,4 +928,8 @@ const QString&SvnItemModel::uniqueIdentifier()const
     return m_Data->m_Uid;
 }
 
+void SvnItemModel::slotNotifyMessage(const QString&msg)
+{
+    kDebug()<<msg;
+}
 #include "svnitemmodel.moc"
