@@ -1,6 +1,6 @@
 /*
  * Port for usage with qt-framework and development for kdesvn
- * (C) 2005-2007 by Rajko Albrecht (ral@alwins-world.de)
+ * Copyright (C) 2005-2009 by Rajko Albrecht (ral@alwins-world.de)
  * http://kdesvn.alwins-world.de
  */
 /*
@@ -31,20 +31,20 @@
 #pragma warning( disable: 4786 )// debug symbol truncated
 #endif
 // svncpp
-#include "svnqt/client_impl.hpp"
+#include "svnqt/client_impl.h"
 
 // subversion api
 #include "svn_client.h"
 
-#include "svnqt/exception.hpp"
-#include "svnqt/pool.hpp"
-#include "svnqt/targets.hpp"
-#include "svnqt/svnqt_defines.hpp"
-#include "svnqt/stringarray.hpp"
-#include "svnqt/client_parameter.hpp"
-#include "svnqt/client_commit_parameter.hpp"
+#include "svnqt/exception.h"
+#include "svnqt/pool.h"
+#include "svnqt/targets.h"
+#include "svnqt/svnqt_defines.h"
+#include "svnqt/stringarray.h"
+#include "svnqt/client_parameter.h"
+#include "svnqt/client_commit_parameter.h"
 
-#include "svnqt/helper.hpp"
+#include "svnqt/helper.h"
 
 namespace svn
 {
@@ -199,9 +199,6 @@ namespace svn
     apr_array_header_t *apr_revisions = apr_array_make (apr_pool,
                       path.size(),
                       sizeof (svn_revnum_t));
-    if (depth==DepthUnknown) {
-        depth=DepthInfinity;
-    }
 #if ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 5)) || (SVN_VER_MAJOR > 1)
     error = svn_client_update3(&apr_revisions,path.array(pool),revision,internal::DepthToSvn(depth),sticky_depth,ignore_externals,allow_unversioned,*m_context,pool);
 #else
