@@ -1,6 +1,6 @@
 /*
  * Port for usage with qt-framework and development for kdesvn
- * (C) 2005-2008 by Rajko Albrecht
+ * Copyright (C) 2005-2009 by Rajko Albrecht (ral@alwins-world.de)
  * http://kdesvn.alwins-world.de
  */
 /*
@@ -33,8 +33,8 @@
 
 
 // svncpp
-#include "svnqt/client_impl.hpp"
-#include "svnqt/helper.hpp"
+#include "svnqt/client_impl.h"
+#include "svnqt/helper.h"
 
 // Subversion api
 #include "svn_client.h"
@@ -42,105 +42,20 @@
 #include "svn_path.h"
 //#include "svn_utf.h"
 
-#include "svnqt/dirent.hpp"
-#include "svnqt/exception.hpp"
-#include "svnqt/pool.hpp"
-#include "svnqt/status.hpp"
-#include "svnqt/targets.hpp"
-#include "svnqt/info_entry.hpp"
-#include "svnqt/url.hpp"
-#include "svnqt/svnqt_defines.hpp"
-#include "svnqt/context_listener.hpp"
+#include "svnqt/dirent.h"
+#include "svnqt/exception.h"
+#include "svnqt/pool.h"
+#include "svnqt/status.h"
+#include "svnqt/targets.h"
+#include "svnqt/info_entry.h"
+#include "svnqt/url.h"
+#include "svnqt/svnqt_defines.h"
+#include "svnqt/context_listener.h"
+#include "svnqt/client_parameter.h"
 
 namespace svn
 {
 
-#if ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 5)) || (SVN_VER_MAJOR > 1)
-    static svn_error_t *
-    logReceiver2(
-                 void*baton,
-                 svn_log_entry_t *log_entry,
-                 apr_pool_t * pool
-                )
-    {
-        Q_UNUSED(pool);
-        Client_impl::sBaton * l_baton = (Client_impl::sBaton*)baton;
-        LogEntries * entries =
-                (LogEntries *) l_baton->m_data;
-        QLIST<QLONG>*rstack=
-                (QLIST<QLONG>*)l_baton->m_revstack;
-        Context*l_context = l_baton->m_context;
-        svn_client_ctx_t*ctx = l_context->ctx();
-        if (ctx&&ctx->cancel_func) {
-            SVN_ERR(ctx->cancel_func(ctx->cancel_baton));
-        }
-        if (! SVN_IS_VALID_REVNUM(log_entry->revision))
-        {
-            if (rstack&&rstack->size()>0) {
-                rstack->pop_front();
-            }
-            return SVN_NO_ERROR;
-        }
-        entries->insert (entries->begin (), LogEntry (log_entry));
-        if (rstack) {
-            entries->first().m_MergedInRevisions=(*rstack);
-            if (log_entry->has_children) {
-                rstack->push_front(log_entry->revision);
-            }
-        }
-        return SVN_NO_ERROR;
-    }
-#else
-  static svn_error_t *
-  logReceiver (
-                 void *baton,
-                 apr_hash_t * changedPaths,
-                 svn_revnum_t rev,
-                 const char *author,
-                 const char *date,
-                 const char *msg,
-                 apr_pool_t * pool
-              )
-  {
-    Client_impl::sBaton * l_baton = (Client_impl::sBaton*)baton;
-    LogEntries * entries =
-      (LogEntries *) l_baton->m_data;
-
-    /* check every loop for cancel of operation */
-    Context*l_context = l_baton->m_context;
-    svn_client_ctx_t*ctx = l_context->ctx();
-    if (ctx&&ctx->cancel_func) {
-        SVN_ERR(ctx->cancel_func(ctx->cancel_baton));
-    }
-    entries->insert (entries->begin (), LogEntry (rev, author, date, msg));
-    if (changedPaths != NULL)
-    {
-      LogEntry &entry = entries->front ();
-
-      for (apr_hash_index_t *hi = apr_hash_first (pool, changedPaths);
-           hi != NULL;
-           hi = apr_hash_next (hi))
-      {
-        const void *pv;
-        void *val;
-        apr_hash_this (hi, &pv, NULL, &val);
-
-        svn_log_changed_path_t *log_item = reinterpret_cast<svn_log_changed_path_t *> (val);
-        const char* path = reinterpret_cast<const char*>(pv);
-
-        entry.changedPaths.push_back (
-              LogChangePathEntry (path,
-                                  log_item->action,
-                                  log_item->copyfrom_path,
-                                  log_item->copyfrom_rev) );
-
-      }
-    }
-
-    return SVN_NO_ERROR;
-  }
-#endif
-
 #if ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 5)) || (SVN_VER_MAJOR > 1)
   static svn_error_t *
           logMapReceiver2(
@@ -167,7 +82,7 @@ namespace svn
           }
           return SVN_NO_ERROR;
       }
-      (*entries)[log_entry->revision]=LogEntry (log_entry);
+      (*entries)[log_entry->revision]=LogEntry (log_entry,(l_baton->excludeList?*(l_baton->excludeList):svn::StringArray()));
       /// @TODO insert it into last logentry
       if (rstack) {
           (*entries)[log_entry->revision].m_MergedInRevisions=(*rstack);
@@ -202,6 +117,7 @@ namespace svn
     if (changedPaths != NULL)
     {
       LogEntry &entry = (*entries)[rev];
+      bool blocked = false;
 
       for (apr_hash_index_t *hi = apr_hash_first (pool, changedPaths);
            hi != NULL;
@@ -214,12 +130,23 @@ namespace svn
         svn_log_changed_path_t *log_item = reinterpret_cast<svn_log_changed_path_t *> (val);
         const char* path = reinterpret_cast<const char*>(pv);
 
-        entry.changedPaths.push_back (
-              LogChangePathEntry (path,
-                                  log_item->action,
-                                  log_item->copyfrom_path,
-                                  log_item->copyfrom_rev) );
-
+        blocked = false;
+        if (l_baton->excludeList) {
+            QString _p(path);
+            for (int _exnr=0;_exnr < l_baton->excludeList.size();++_exnr) {
+                if (_p.startsWith(l_baton->excludeList[_exnr])) {
+                    blocked=true;
+                    break;
+                }
+            }
+        }
+        if (!blocked) {
+            entry.changedPaths.push_back (
+                LogChangePathEntry (path,
+                                    log_item->action,
+                                    log_item->copyfrom_path,
+                                    log_item->copyfrom_rev) );
+        }
       }
     }
 
@@ -228,12 +155,21 @@ namespace svn
 #endif
 
   struct StatusEntriesBaton {
+    StatusEntries entries;
     apr_pool_t* pool;
-    apr_hash_t* hash;
     Context*m_Context;
-    StatusEntriesBaton() {
+    StatusEntriesBaton():entries() {
+        pool = 0;
+        m_Context = 0;
+    }
+  };
+
+  struct InfoEntriesBaton {
+    InfoEntries entries;
+    apr_pool_t* pool;
+    Context*m_Context;
+    InfoEntriesBaton():entries() {
         pool = 0;
-        hash = 0;
         m_Context = 0;
     }
   };
@@ -243,7 +179,7 @@ namespace svn
                             const svn_info_t*info,
                             apr_pool_t *)
   {
-    StatusEntriesBaton* seb = (StatusEntriesBaton*)baton;
+    InfoEntriesBaton* seb = (InfoEntriesBaton*)baton;
     if (seb->m_Context) {
         /* check every loop for cancel of operation */
         Context*l_context = seb->m_Context;
@@ -252,101 +188,92 @@ namespace svn
             SVN_ERR(ctx->cancel_func(ctx->cancel_baton));
         }
     }
-    path = apr_pstrdup (seb->pool, path);
-    InfoEntry*e = new InfoEntry(info,path);
-    apr_hash_set (seb->hash, path, APR_HASH_KEY_STRING, e);
+    seb->entries.push_back(InfoEntry(info,path));
     return NULL;
   }
 
+#if ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 6)) || (SVN_VER_MAJOR > 1)
+  static svn_error_t* StatusEntriesFunc (void *baton,
+                                 const char *path,
+                                 svn_wc_status2_t *status,
+                                 apr_pool_t *pool)
+  {
+        // use own pool - the parameter will cleared between loops!
+        Q_UNUSED(pool);
+        StatusEntriesBaton* seb = (StatusEntriesBaton*)baton;
+        if (seb->m_Context) {
+            /* check every loop for cancel of operation */
+            Context*l_context = seb->m_Context;
+            svn_client_ctx_t*ctx = l_context->ctx();
+            if (ctx&&ctx->cancel_func) {
+                SVN_ERR(ctx->cancel_func(ctx->cancel_baton));
+            }
+        }
+
+        seb->entries.push_back (StatusPtr(new Status(path,status)));
+        return NULL;
+  }
+#else
   static void StatusEntriesFunc (void *baton,
                                  const char *path,
                                  svn_wc_status2_t *status)
   {
-    svn_wc_status2_t* stat;
       StatusEntriesBaton* seb = (StatusEntriesBaton*)baton;
-
-      path = apr_pstrdup (seb->pool, path);
-      stat = svn_wc_dup_status2 (status, seb->pool);
-      apr_hash_set (seb->hash, path, APR_HASH_KEY_STRING, stat);
+      seb->entries.push_back (StatusPtr(new Status(path,status)));
   }
+#endif
 
   static StatusEntries
-  localStatus (const Path& path,
-               Depth depth,
-               const bool get_all,
-               const bool update,
-               const bool no_ignore,
-               const bool hide_externals,
-               const StringArray & changelists,
+  localStatus (const StatusParameter&params,
                Context * context)
   {
     svn_error_t *error;
     StatusEntries entries;
-    apr_hash_t *status_hash;
     svn_revnum_t revnum;
     Revision rev (Revision::HEAD);
     Pool pool;
     StatusEntriesBaton baton;
 
-    status_hash = apr_hash_make (pool);
-    baton.hash = status_hash;
     baton.pool = pool;
+
 #if ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 5)) || (SVN_VER_MAJOR > 1)
+
+#if ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 6)) || (SVN_VER_MAJOR > 1)
+    error = svn_client_status4 (
+#else
     error = svn_client_status3 (
+#endif
         &revnum,           // revnum
-        path.path().TOUTF8(),         // path
+        params.path().path().TOUTF8(),         // path
         rev,
         StatusEntriesFunc, // status func
         &baton,            // status baton
-        internal::DepthToSvn(depth), // see svn::Depth
-        get_all,           // get all not only interesting
-        update,            // check for updates
-        no_ignore,         // hide ignored files or not
-        hide_externals,    // hide external
-        changelists.array(pool),
+        internal::DepthToSvn(params.depth()), // see svn::Depth
+        params.all(),           // get all not only interesting
+        params.update(),            // check for updates
+        params.noIgnore(),         // hide ignored files or not
+        params.ignoreExternals(),    // hide external
+        params.changeList().array(pool),
         *context,          //client ctx
         pool);
 #else
-    Q_UNUSED(changelists);
     error = svn_client_status2 (
       &revnum,      // revnum
-      path.path().TOUTF8(),         // path
+      params.path().path().TOUTF8(),         // path
       rev,
       StatusEntriesFunc, // status func
       &baton,        // status baton
-      (depth==DepthInfinity), //recurse
-      get_all,       // get all not only interesting
-      update,        // check for updates
-      no_ignore,     // hide ignored files or not
-      hide_externals, // hide external
+      (params.depth()==DepthInfinity), //recurse
+      params.all(),       // get all not only interesting
+      params.update(),        // check for updates
+      params.noIgnore(),     // hide ignored files or not
+      params.ignoreExternals(), // hide external
       *context,    //client ctx
       pool);
 #endif
 
-    if (error!=NULL)
-    {
-      throw ClientException (error);
-    }
-
-    apr_array_header_t *statusarray =
-      svn_sort__hash (status_hash, svn_sort_compare_items_as_paths,
-                            pool);
-    int i;
-
-    /* Loop over array, printing each name/status-structure */
-    for (i = 0; i < statusarray->nelts; ++i)
-    {
-      const svn_sort__item_t *item;
-      const char *filePath;
-      svn_wc_status2_t *status = NULL;
-
-      item = &APR_ARRAY_IDX (statusarray, i, const svn_sort__item_t);
-      status = (svn_wc_status2_t *) item->value;
-
-      filePath = (const char *) item->key;
-      entries.push_back (StatusPtr(new Status(filePath, status)));
-    }
-    return entries;
+    Client_impl::checkErrorThrow(error);
+    return baton.entries;
   }
 
   static StatusPtr
@@ -366,49 +293,33 @@ namespace svn
 
   static StatusEntries
   remoteStatus (Client * client,
-                const Path& path,
-                Depth depth,
-                const bool ,
-                const bool ,
-                const bool ,
-                const Revision revision,
-                Context * ,
-                bool detailed_remote)
+                const StatusParameter&params,
+                Context *)
   {
-    DirEntries dirEntries = client->list(path, revision, revision, depth,detailed_remote);
+    DirEntries dirEntries = client->list(params.path(), params.revision(), params.revision(), params.depth(),params.detailedRemote());
     DirEntries::const_iterator it;
 
     StatusEntries entries;
-    QString url = path.path();
+    QString url = params.path().path();
     url+=QString::FROMUTF8("/");
 
-    for (it = dirEntries.begin (); it != dirEntries.end (); it++)
+    for (it = dirEntries.begin (); it != dirEntries.end (); ++it)
     {
         DirEntryPtr dirEntry = *it;
         if (dirEntry->name().isEmpty())
             continue;
-        entries.push_back(dirEntryToStatus (path, dirEntry));
+        entries.push_back(dirEntryToStatus (params.path(), dirEntry));
     }
     return entries;
   }
 
   StatusEntries
-  Client_impl::status (const Path& path,
-                  Depth depth,
-                  const bool get_all,
-                  const bool update,
-                  const bool no_ignore,
-                  const Revision revision,
-                  bool detailed_remote,
-                  const bool hide_externals,
-                  const StringArray & changelists) throw (ClientException)
+  Client_impl::status (const StatusParameter&params) throw (ClientException)
   {
-    if (Url::isValid (path.path())) {
-        return remoteStatus (this, path, depth, get_all, update,
-                            no_ignore,revision,m_context,detailed_remote);
+    if (Url::isValid (params.path().path())) {
+        return remoteStatus (this, params,m_context);
     } else {
-        return localStatus (path, depth, get_all, update,
-                            no_ignore, hide_externals,changelists, m_context);
+        return localStatus (params,m_context);
     }
   }
 
@@ -416,18 +327,20 @@ namespace svn
   localSingleStatus (const Path& path, Context * context,bool update=false)
   {
     svn_error_t *error;
-    apr_hash_t *status_hash;
     Pool pool;
     StatusEntriesBaton baton;
     svn_revnum_t revnum;
     Revision rev (Revision::HEAD);
 
-    status_hash = apr_hash_make( pool );
-    baton.hash = status_hash;
     baton.pool = pool;
 
 #if ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 5)) || (SVN_VER_MAJOR > 1)
+
+#if ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 6)) || (SVN_VER_MAJOR > 1)
+    error = svn_client_status4 (
+#else
     error = svn_client_status3 (
+#endif
         &revnum,           // revnum
         path.path().TOUTF8(),         // path
         rev,
@@ -435,9 +348,9 @@ namespace svn
         &baton,            // status baton
         svn_depth_empty, // not recurse
         true,           // get all not only interesting
-        update,            // check for updates
-        false,         // hide ignored files or not
-        false,    // hide external
+        update,         // check for updates
+        false,          // hide ignored files or not
+        false,          // hide external
         0,
         *context,          //client ctx
         pool);
@@ -456,23 +369,12 @@ namespace svn
       *context,    //client ctx
       pool);
 #endif
-    if (error != NULL)
-    {
-      throw ClientException (error);
+    Client_impl::checkErrorThrow(error);
+    if (baton.entries.size()==0) {
+        return StatusPtr(new Status());
     }
 
-    apr_array_header_t *statusarray =
-      svn_sort__hash (status_hash, svn_sort_compare_items_as_paths,
-                            pool);
-    const svn_sort__item_t *item;
-    const char *filePath;
-    svn_wc_status2_t *status = NULL;
-
-    item = &APR_ARRAY_IDX (statusarray, 0, const svn_sort__item_t);
-    status = (svn_wc_status2_t *) item->value;
-    filePath = (const char *) item->key;
-
-    return StatusPtr(new Status (filePath, status));
+    return baton.entries[0];
   };
 
   static StatusPtr
@@ -495,153 +397,74 @@ namespace svn
   }
 
   bool
-  Client_impl::log (const Path& path, const Revision & revisionStart,
-       const Revision & revisionEnd,
-       LogEntriesMap&log_target,
-       const Revision & revisionPeg,
-       bool discoverChangedPaths,
-       bool strictNodeHistory,int limit,
-       bool include_merged_revisions,
-       const StringArray&revprops
-                   ) throw (ClientException)
+  Client_impl::log (const LogParameter&params,LogEntriesMap&log_target) throw (ClientException)
   {
-    Targets target(path);
     Pool pool;
     sBaton l_baton;
     QLIST<QLONG> revstack;
     l_baton.m_context=m_context;
     l_baton.m_data = &log_target;
     l_baton.m_revstack = &revstack;
+    l_baton.excludeList = & (params.excludeList());
     svn_error_t *error;
 
-#if ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 5)) || (SVN_VER_MAJOR > 1)
+#if ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 6)) || (SVN_VER_MAJOR > 1)
+    error = svn_client_log5 (
+        params.targets().array (pool),
+        params.peg().revision(),
+        revListToHeader(params.revisions(),pool),
+        params.limit(),
+        params.discoverChangedPathes() ? 1 : 0,
+        params.strictNodeHistory() ? 1 : 0,
+        params.includeMergedRevisions()?1:0,
+        params.revisionProperties().array(pool),
+        logMapReceiver2,
+        &l_baton,
+        *m_context, // client ctx
+        pool);
+#elif ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 5))
     error = svn_client_log4 (
-                target.array (pool),
-                revisionPeg.revision(),
-                revisionStart.revision (),
-                revisionEnd.revision (),
-                limit,
-                discoverChangedPaths ? 1 : 0,
-                strictNodeHistory ? 1 : 0,
-                include_merged_revisions?1:0,
-                revprops.array(pool),
-                logMapReceiver2,
-                &l_baton,
-                *m_context, // client ctx
-                pool);
-#elif ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 4)) || (SVN_VER_MAJOR > 1)
-    Q_UNUSED(include_merged_revisions);
-    Q_UNUSED(revprops);
-
-    error = svn_client_log3 (
-      target.array (pool),
-      revisionPeg.revision(),
-      revisionStart.revision (),
-      revisionEnd.revision (),
-      limit,
-      discoverChangedPaths ? 1 : 0,
-      strictNodeHistory ? 1 : 0,
-      logMapReceiver,
-      &l_baton,
-      *m_context, // client ctx
-      pool);
-#else
-    Q_UNUSED(include_merged_revisions);
-    Q_UNUSED(revprops);
-    Q_UNUSED(revisionPeg);
-
-    error = svn_client_log2 (
-      target.array (pool),
-      revisionStart.revision (),
-      revisionEnd.revision (),
-      limit,
-      discoverChangedPaths ? 1 : 0,
-      strictNodeHistory ? 1 : 0,
-      logMapReceiver,
-      &l_baton,
-      *m_context, // client ctx
-      pool);
-#endif
-    if (error != NULL)
-    {
-      throw ClientException (error);
-    }
-    return true;
-  }
-
-  LogEntriesPtr
-  Client_impl::log (const Path& path, const Revision & revisionStart,
-                    const Revision & revisionEnd, const Revision & revisionPeg,
-                    bool discoverChangedPaths,
-                    bool strictNodeHistory,int limit,
-                    bool include_merged_revisions,
-                    const StringArray&revprops
-                   ) throw (ClientException)
-  {
-    Targets target(path);
-    Pool pool;
-    LogEntriesPtr entries = LogEntriesPtr(new LogEntries ());
-    QLIST<QLONG> revstack;
-    sBaton l_baton;
-    l_baton.m_context=m_context;
-    l_baton.m_data = entries;
-    l_baton.m_revstack = &revstack;
-
-    svn_error_t *error;
-#if ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 5)) || (SVN_VER_MAJOR > 1)
-    error = svn_client_log4 (
-        target.array (pool),
-        revisionPeg.revision(),
-        revisionStart.revision (),
-        revisionEnd.revision (),
-        limit,
-        discoverChangedPaths ? 1 : 0,
-        strictNodeHistory ? 1 : 0,
-        include_merged_revisions?1:0,
-        revprops.array(pool),
-        logReceiver2,
+        params.targets().array (pool),
+        params.peg().revision(),
+        params.revisionRange().first.revision (),
+        params.revisionRange().second.revision (),
+        params.limit(),
+        params.discoverChangedPathes() ? 1 : 0,
+        params.strictNodeHistory() ? 1 : 0,
+        params.includeMergedRevisions()?1:0,
+        params.revisionProperties().array(pool),
+        logMapReceiver2,
         &l_baton,
         *m_context, // client ctx
         pool);
 #elif ((SVN_VER_MAJOR == 1) && (SVN_VER_MINOR >= 4)) || (SVN_VER_MAJOR > 1)
-    Q_UNUSED(include_merged_revisions);
-    Q_UNUSED(revprops);
-
     error = svn_client_log3 (
-      target.array (pool),
-      revisionPeg.revision(),
-      revisionStart.revision (),
-      revisionEnd.revision (),
-      limit,
-      discoverChangedPaths ? 1 : 0,
-      strictNodeHistory ? 1 : 0,
-      logReceiver,
-      &l_baton,
-      *m_context, // client ctx
-      pool);
+        params.targets().array (pool),
+        params.peg().revision(),
+        params.revisionRange().first.revision (),
+        params.revisionRange().second.revision (),
+        params.limit(),
+        params.discoverChangedPathes() ? 1 : 0,
+        params.strictNodeHistory() ? 1 : 0,
+        logMapReceiver,
+        &l_baton,
+        *m_context, // client ctx
+        pool);
 #else
-    Q_UNUSED(include_merged_revisions);
-    Q_UNUSED(revprops);
-    Q_UNUSED(revisionPeg);
-
     error = svn_client_log2 (
-      target.array (pool),
-      revisionStart.revision (),
-      revisionEnd.revision (),
-      limit,
-      discoverChangedPaths ? 1 : 0,
-      strictNodeHistory ? 1 : 0,
-      logReceiver,
-      &l_baton,
-      *m_context, // client ctx
-      pool);
+        params.targets().array (pool),
+        params.revisionRange().first.revision (),
+        params.revisionRange().second.revision (),
+        params.limit(),
+        params.discoverChangedPathes() ? 1 : 0,
+        params.strictNodeHistory() ? 1 : 0,
+        logMapReceiver,
+        &l_baton,
+        *m_context, // client ctx
+        pool);
 #endif
-    if (error != NULL)
-    {
-      throw ClientException (error);
-    }
-
-    return entries;
+    checkErrorThrow(error);
+    return true;
   }
 
   InfoEntries
@@ -653,14 +476,10 @@ namespace svn
                     ) throw (ClientException)
   {
 
-    InfoEntries ientries;
     Pool pool;
     svn_error_t *error = NULL;
-    StatusEntriesBaton baton;
-    apr_hash_t *status_hash;
+    InfoEntriesBaton baton;
 
-    status_hash = apr_hash_make (pool);
-    baton.hash = status_hash;
     baton.pool = pool;
     baton.m_Context=m_context;
     svn_opt_revision_t pegr;
@@ -671,9 +490,7 @@ namespace svn
     error = svn_opt_parse_path(&pegr, &truepath,
                                  _buf,
                                  pool);
-    if (error != NULL)
-      throw ClientException (error);
-
+    checkErrorThrow(error);
     if (peg_revision.kind() == svn_opt_revision_unspecified) {
         if ((svn_path_is_url (_p.cstr())) && (pegr.kind == svn_opt_revision_unspecified)) {
             pegr.kind = svn_opt_revision_head;
@@ -706,27 +523,8 @@ namespace svn
                       pool);
 #endif
 
-    if (error != NULL)
-      throw ClientException (error);
-
-    apr_array_header_t *statusarray =
-      svn_sort__hash (status_hash, svn_sort_compare_items_as_paths,
-                            pool);
-    int i;
-
-    /* Loop over array, printing each name/status-structure */
-    for (i=0; i< statusarray->nelts; ++i)
-    {
-      const svn_sort__item_t *item;
-      InfoEntry*e = NULL;
-      item = &APR_ARRAY_IDX (statusarray, i, const svn_sort__item_t);
-      e = (InfoEntry *) item->value;
-      ientries.push_back(*e);
-      delete e;
-    }
-    if (error != NULL)
-      throw ClientException (error);
-    return ientries;
+    checkErrorThrow(error);
+    return baton.entries;
   }
 
 }
