@@ -1,6 +1,6 @@
 /*
  * Port for usage with qt-framework and development for kdesvn
- * (C) 2005-2007 by Rajko Albrecht (ral@alwins-world.de)
+ * Copyright (C) 2005-2009 by Rajko Albrecht (ral@alwins-world.de)
  * http://kdesvn.alwins-world.de
  */
 /*
@@ -29,9 +29,9 @@
  */
 
 // svncpp
-#include "svnqt/dirent.hpp"
-#include "svnqt/lock_entry.hpp"
-#include "svnqt/svnqt_defines.hpp"
+#include "svnqt/dirent.h"
+#include "svnqt/lock_entry.h"
+#include "svnqt/svnqt_defines.h"
 
 #include <qstring.h>
 
